#if CLANG_MAJOR < 16
#define LLVM_OLDER_THAN_16_0 1
#endif

#if CLANG_MAJOR < 15
#define LLVM_OLDER_THAN_15_0 1
#endif

#if CLANG_MAJOR < 14
#define LLVM_OLDER_THAN_14_0 1
#endif

#if CLANG_MAJOR < 13
#define LLVM_OLDER_THAN_13_0 1
#endif

#if CLANG_MAJOR < 12
#define LLVM_OLDER_THAN_12_0 1
#endif

#if CLANG_MAJOR < 11
#define LLVM_OLDER_THAN_11_0 1
#endif
